package de.w665.sharepulse.service;

import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.crypto.SecretKey;
import java.util.Base64;
import java.util.Date;

@Slf4j
@Service
public class AuthenticationService {
    @Value("${secureapi.jwt.secret}")
    private String secretString;

    @Value("${secureapi.jwt.expiration}")
    private long expirationTime; // in milliseconds

    private SecretKey secretKey;

    @PostConstruct
    public void init() {
        System.out.println("Initializing secret key");
        byte[] encodedKey = Base64.getEncoder().encode(secretString.getBytes()); // encode the secret key
        this.secretKey = Keys.hmacShaKeyFor(encodedKey);
    }

    public String authenticate(String username, String password) {
        // validate user credentials with repository and password hash algorithm

        return generateToken(username);
        // throw new RuntimeException("User authentication failed");
    }

    private String generateToken(String username) {
        long nowMillis = System.currentTimeMillis();
        Date now = new Date(nowMillis);
        Date expiryDate = new Date(nowMillis + expirationTime);

        return Jwts.builder()
                .subject(username)
                .issuedAt(now)
                .expiration(expiryDate)
                .signWith(secretKey)
                .compact();
    }
}
